"""
RSA_tool module

eng: Module for fast calculating large primes, and RSA encryption and decryption of data of any size.

rus: Модуль сделан для быстрого вычисления простых чисел, и шифрования/расшифрования данных любого размера по алгоритму RSA


"""
from .RSA_tool import RSA


__author__ = "Nikita Grigorev, Andrey Ignatov"
__date__ = "2023-03-22"
__version__ = "1.1"


import random
from .prime_numbers import *
from .encryption import *


class RSA:
    def __init__(self):
        pass

    @classmethod
    def get_bit_keys(cls, col_bits, e=65577):
        '''
        eng:this function generates keys for RSA encryption

        rus:данная функция генерирует ключи для шифрования RSA
        :param col_bits - eng: enter the size of the desired keys
        rus: введите размер желаемых ключей:
        :param e - eng: this is the public key, it is recommended to leave it like this
        rus: это открытый ключ рекомендуется его так оставить:
        :return eng: public key, private key, module
        rus: публичный ключ, приватный ключ, модуль:
        '''
        p = PrimeNumberTool.find_prime_number(random.randint(2 ** (col_bits // 2), 2 ** ((col_bits // 2) + 1)))
        q = PrimeNumberTool.find_prime_number(random.randint(2 ** (col_bits // 2) - 1, 2 ** (col_bits // 2)))
        return gen_keys(p, q, e)

    @classmethod
    def block_encrypt(cls, m, e, n):
        '''
        eng: this function effectively encrypts any amount of information
        rus: данная функция эффективно шифрует любое количество информации
        :param m - message/сообщение:
        :param e - public key/публичный ключ:
        :param n - module/ модуль:
        :return encrypted message/зашифрованное сообщение:
        '''
        max_len_block = len(str(n)) - 2
        block = ''
        arr_blocks = []
        for i in m:
            str_i = str(ord(i))
            while len(str_i) < 4:
                str_i = '0' + str_i
            if len(block + str_i) <= max_len_block:
                block += str_i
            else:
                arr_blocks.append('1' + block)
                block = str_i
        arr_blocks.append('1' + block)
        for i in range(len(arr_blocks)):
            arr_blocks[i] = pow(int(arr_blocks[i]), e, n)
        return arr_blocks

    @classmethod
    def block_decrypt(cls, c, d, n):
        '''
        eng: this function effectively decrypt any amount of information
        rus: данная функция эффективно расшифровывает любое количество информации
        :param c - encrypted message/ зашифрованное сообщение:
        :param d - private key / закрытый ключь:
        :param n - module/ модуль:
        :return decrypted message/расшифрованное сообщение:
        '''
        for i in range(len(c)):
            c[i] = pow(int(c[i]), d, n)
        s = ''
        for i in c:
            tmp = str(i)[1:]
            for j in range(0, len(tmp), 4):
                s += chr(int((tmp[j] + tmp[j + 1] + tmp[j + 2] + tmp[j + 3])))
        return s





import numpy.random
import RSA_tool as my_RSA


def test():
    m = numpy.random.randint(13, 1000, 20)
    isNorm = True
    for i in m:
        i = int(i)
        e, d, n = my_RSA.RSA.get_bit_keys(256)
        if (pow(pow(i, e, n), d, n)) != i:
            print("error")
            isNorm = False
            break
    if isNorm:
        print("eee i feel good")


def test_calculating_keys():
    e, d, n = my_RSA.RSA.get_bit_keys(1024)
    m = 'hello world'
    print(n)
    c = my_RSA.RSA.block_encrypt(m, e, n)
    print(c)
    m = my_RSA.RSA.block_decrypt(c, d, n)
    print(m)

def main():
    test_calculating_keys()



if __name__ == '__main__':
    main()



import math
import random


class PrimeNumberTool:
    def __init__(self):
        pass

    @classmethod
    def BPSW(cls, n):
        '''
        eng: this function checks if a number is prime,this is a modern algorithm that almost does not find pseudoprimes

        rus: эта функция проверяет число на простору, это современный алгоритм который почти не находит псевдопростые числа
        :param n:
        :return bool:
        '''
        if n == 2:
            return True
        if not n & 1:
            return False
        d = n - 1
        while d & 1 == 0:
            d >>= 1
        for a in [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37]:
            x = pow(a, d, n)
            if x == 1 or x == n - 1:
                continue
            for _ in range(100):
                x = pow(x, 2, n)
                if x == 1:
                    return False
                if x == n - 1:
                    a = 0
                    break
            if a:
                return False
        return True

    @classmethod
    def factorization(cls, n):
        '''
        eng: this function factorizes a number

        rus: эта функция факторизует число
        :param n:
        :return first divisor:
        '''
        if n % 2 == 0:
            print(2)
        for i in range(3, math.isqrt(n) + 2, 2):
            if n % i == 0:
                return i

    @classmethod
    def find_prime_number(cls, n):
        '''
        eng: this function goes through spirals with which contain a significant percentage of prime numbers and finds the nearest prime number to the given

        rus: данная функция идёт по спиралям с которые содержат значительный процент простых чисел и находит ближайшее простое число к данному
        :param n:
        :return:
        '''
        n = n + (6 - n % 6)
        i = n
        while True:
            if cls.BPSW(i + 1):
                return i + 1
            elif cls.BPSW(i + 5):
                return i + 5
            i += 6
